/* tslint:disable:no-console */
/**
 * @module SSBIntegration
 */ /** for typedoc */

const { ipcRenderer, crashReporter, webFrame } = require('electron');

function receiveMessage(event) {
  const originURL = new URL(event.origin);
  const isOriginMatch = originURL.hostname.endsWith('slack.com');
  const isDataExpected = !!event.data && !!event.data.type;

  if (!isOriginMatch) {
    console.log(`origin doesn't match`, { origin: event.origin });
    return;
  }

  if (!isDataExpected) {
    //do not log anything, webapp have some cases postmessage doesn't belong to our flow
    return;
  }

  try {
    const { type, args } = event.data;
    switch (type) {
      case 'didFinishLoading':
        ipcRenderer.send('SLACK_DID_FINISH_LOADING');
        break;
      case 'didSignIn':
        ipcRenderer.send('SLACK_DID_SIGN_IN', ...args);
        break;
    }
  } catch (e) {
    console.log(e);
  }
}
window.addEventListener('message', receiveMessage, false);

// Assign preload script proxies to preload script's ipc
const preloadTemplate = `
window.desktop = window.winssb = {
  app: { didFinishLoading: () => { window.postMessage({ type: 'didFinishLoading' }, window.location.href) } },
  teams: { didSignIn: (...args) => { window.postMessage({ type: 'didSignIn', args: args }, window.location.href) } }
};`;

webFrame.executeJavaScript(preloadTemplate);

const allowedMetadata = ['loadSettings', 'crashReporterOpts'];
const assignMetadata = () => {
  try {
    const metadataArgs = process.argv.find((x) => x.includes('slack_preload_metadata_arguments')) || '{}';
    const metadataJson = JSON.parse(metadataArgs);
    Object.keys(metadataJson)
      .filter((key) => allowedMetadata.includes(key))
      .forEach((key) => window[key] = metadataJson[key]);
  } catch (e) {
    console.log(e); //tslint:disable-line:no-console
  }
};
assignMetadata();

if (window.crashReporterOpts) {
  try {
    crashReporter.start(window.crashReporterOpts);
  } catch (error) {
    console.error('CrashReporter: Unable to start crash report service', { error });
  }
}
